@@ -22,6 +22,7 @@ __all__ = [
     'Teams',
     'Tournaments',
     'Users',
+    'Messaging'
 ]
 
 # Base URL for the API
@@ -100,6 +101,7 @@ class Client(BaseClient):
         self.broadcasts = Broadcasts(session, base_url)
         self.simuls = Simuls(session, base_url)
         self.studies = Studies(session, base_url)
+        self.messaging = Messaging(session, base_url)
 
 
 class Account(BaseClient):
@@ -180,17 +182,21 @@ class Users(BaseClient):
         return self._r.get(path, params=params, fmt=NDJSON, stream=True,
                            converter=models.PuzzleActivity.convert)
 
-    def get_realtime_statuses(self, *user_ids):
+    def get_realtime_statuses(self, *user_ids, with_game_ids=False):
         """Get the online, playing, and streaming statuses of players.
 
         Only id and name fields are returned for offline users.
 
         :param user_ids: one or more user IDs (names)
+        :param bool with_game_ids: wether to return or not the ID of the game being played
         :return: statuses of given players
         :rtype: list
         """
         path = 'api/users/status'
-        params = {'ids': ','.join(user_ids)}
+        params = {
+            'ids': ','.join(user_ids),
+            'withGameIds': with_game_ids
+        }
         return self._r.get(path, params=params)
 
     def get_all_top_10(self):
@@ -403,13 +409,15 @@ class Games(FmtClient):
 
     def export_by_player(self, username, as_pgn=None, since=None, until=None,
                          max=None, vs=None, rated=None, perf_type=None,
-                         color=None, analysed=None, moves=None, tags=None,
-                         evals=None, opening=None):
+                         color=None, analysed=None, moves=None,
+                         pgn_in_json=None, tags=None, clocks=None, evals=None,
+                         opening=None, ongoing=None, finished=None, players=None,
+                         sort=None, literate=None):
         """Get games by player.
 
         :param str username: which player's games to return
         :param bool as_pgn: whether to return the game in PGN format
-        :param int since: lowerbound on the game timestamp
+        :param int since: lower bound on the game timestamp
         :param int until: upperbound on the game timestamp
         :param int max: limit the number of games returned
         :param str vs: filter by username of the opponent
@@ -421,11 +429,16 @@ class Games(FmtClient):
         :type color: :class:`~berserk.enums.Color`
         :param bool analysed: filter by analysis availability
         :param bool moves: whether to include the PGN moves
+        :param bool pgnInJson: Include the full PGN within JSON response
         :param bool tags: whether to include the PGN tags
         :param bool clocks: whether to include clock comments in the PGN moves
         :param bool evals: whether to include analysis evaluation comments in
                            the PGN moves when available
         :param bool opening: whether to include the opening name
+        :param bool ongoing: Include ongoing games, last 3 moves omitted
+        :param bool finished: Include finished games
+        :param str players: URL of text file containing real names and ratings for PGN
+        :param str sort: Sort the order of games
         :param bool literate: whether to include literate the PGN
         :return: iterator over the exported games, as JSON or PGN
         """
@@ -440,9 +453,16 @@ class Games(FmtClient):
             'color': color,
             'analysed': analysed,
             'moves': moves,
+            'pgnInJson': pgn_in_json,
             'tags': tags,
+            'clocks': clocks,
             'evals': evals,
             'opening': opening,
+            'ongoing': ongoing,
+            'finished': finished,
+            'players': players,
+            'sort': sort,
+            'literate': literate,
         }
         fmt = PGN if self._use_pgn(as_pgn) else NDJSON
         yield from self._r.get(path, params=params, fmt=fmt, stream=True,
@@ -616,7 +636,7 @@ class Challenges(BaseClient):
                             converter=models.Tournament.convert)
 
     def create_open(self, clock_limit=None, clock_increment=None,
-                    variant=None, position=None):
+                    variant=None, position=None, rated=None, name=None):
         """Create a challenge that any two players can join.
 
         :param int clock_limit: clock initial time (in seconds)
@@ -626,6 +646,10 @@ class Challenges(BaseClient):
         :param position: custom intial position in FEN (variant must be
                          standard and the game cannot be rated)
         :type position: str
+        :param bool: rated: Game is rated and impacts players ratings
+        :param name: Optional name for the challenge, that players will see on
+                     the challenge page.
+        :type name: str
         :return: challenge data
         :rtype: dict
         """
@@ -635,6 +659,8 @@ class Challenges(BaseClient):
             'clock.increment': clock_increment,
             'variant': variant,
             'fen': position,
+            'rated': rated,
+            'name': name,
         }
         return self._r.post(path, json=payload,
                             converter=models.Tournament.convert)
@@ -773,7 +799,7 @@ class Board(BaseClient):
 
         To offer a draw, pass ``accept=True`` and a game ID of an in-progress
         game. To response to a draw offer, pass either ``accept=True`` or
-        ``accept=False`` and the ID of a game in which you have recieved a
+        ``accept=False`` and the ID of a game in which you have received a
         draw offer.
 
         Often, it's easier to call :func:`offer_draw`, :func:`accept_draw`, or
@@ -815,6 +841,53 @@ class Board(BaseClient):
         """
         return self.handle_draw_offer(game_id, False)
 
+    def handle_takeback_offer(self, game_id, accept):
+        """Create, accept, or decline a takeback offer.
+
+        To offer a takeback, pass ``accept=True`` and a game ID of an in-progress
+        game. To response to a takeback offer, pass either ``accept=True`` or
+        ``accept=False`` and the ID of a game in which you have received a
+        takeback offer.
+
+        Often, it's easier to call :func:`offer_takeback`, :func:`accept_takeback`, or
+        :func:`decline_takeback`.
+
+        :param str game_id: ID of an in-progress game
+        :param bool accept: whether to accept
+        :return: True if successful
+        :rtype: bool
+        """
+        accept = "yes" if accept else "no"
+        path = f'/api/board/game/{game_id}/takeback/{accept}'
+        return self._r.post(path)['ok']
+
+    def offer_takeback(self, game_id):
+        """Offer a takeback in the given game.
+
+        :param str game_id: ID of an in-progress game
+        :return: True if successful
+        :rtype: bool
+        """
+        return self.handle_takeback_offer(game_id, True)
+
+    def accept_takeback(self, game_id):
+        """Accept an already offered takeback in the given game.
+
+        :param str game_id: ID of an in-progress game
+        :return: True if successful
+        :rtype: bool
+        """
+        return self.handle_takeback_offer(game_id, True)
+
+    def decline_takeback(self, game_id):
+        """Decline an already offered takeback in the given game.
+
+        :param str game_id: ID of an in-progress game
+        :return: True if successful
+        :rtype: bool
+        """
+        return self.handle_takeback_offer(game_id, False)
+
 
 class Bots(BaseClient):
     """Client for bot-related endpoints."""
@@ -927,7 +1000,7 @@ class Tournaments(FmtClient):
         :rtype: dict
         """
         path = f'api/tournament/{tournament_id}?page={page}'
-        return self._r.get(path, converter=models.Tournaments.convert_values)
+        return self._r.get(path, converter=models.Tournaments.convert_tournament_values)
 
     @deprecated(version='0.11.0', reason='use Tournaments.create_arena or Tournaments.create_swiss instead')
     def create(self, clock_time, clock_increment, minutes, name=None,
@@ -1252,23 +1325,13 @@ class Tournaments(FmtClient):
 class Broadcasts(BaseClient):
     """Broadcast of one or more games."""
 
-    def create(self, name, description, sync_url=None, markdown=None,
-               credit=None, starts_at=None, official=None, throttle=None):
+    def create(self, name, description, markdown=None, official=None):
         """Create a new broadcast.
 
-        .. note::
-
-            ``sync_url`` must be publicly accessible. If not provided, you
-            must periodically push new PGN to update the broadcast manually.
-
         :param str name: name of the broadcast
         :param str description: short description
         :param str markdown: long description
-        :param str sync_url: URL by which Lichess can poll for updates
-        :param str credit: short text to give credit to the source provider
-        :param int starts_at: start time as millis
         :param bool official: DO NOT USE
-        :param int throttle: DO NOT USE
         :return: created tournament info
         :rtype: dict
         """
@@ -1276,12 +1339,8 @@ class Broadcasts(BaseClient):
         payload = {
             'name': name,
             'description': description,
-            'syncUrl': sync_url,
             'markdown': markdown,
-            'credit': credit,
-            'startsAt': starts_at,
             'official': official,
-            'throttle': throttle,
         }
         return self._r.post(path, json=payload,
                             converter=models.Broadcast.convert)
@@ -1297,9 +1356,8 @@ class Broadcasts(BaseClient):
         path = f'broadcast/{slug}/{broadcast_id}'
         return self._r.get(path, converter=models.Broadcast.convert)
 
-    def update(self, broadcast_id, name, description, sync_url, markdown=None,
-               credit=None, starts_at=None, official=None, throttle=None,
-               slug='-'):
+    def update(self, broadcast_id, name, description, markdown=None,
+               official=None, slug='-'):
         """Update an existing broadcast by ID.
 
         .. note::
@@ -1309,12 +1367,8 @@ class Broadcasts(BaseClient):
         :param str broadcast_id: ID of a broadcast
         :param str name: name of the broadcast
         :param str description: short description
-        :param str sync_url: URL by which Lichess can poll for updates
         :param str markdown: long description
-        :param str credit: short text to give credit to the source provider
-        :param int starts_at: start time as millis
         :param bool official: DO NOT USE
-        :param int throttle: DO NOT USE
         :param str slug: slug for SEO
         :return: updated broadcast information
         :rtype: dict
@@ -1323,28 +1377,73 @@ class Broadcasts(BaseClient):
         payload = {
             'name': name,
             'description': description,
-            'syncUrl': sync_url,
             'markdown': markdown,
-            'credit': credit,
-            'startsAt': starts_at,
             'official': official,
-
         }
         return self._r.post(path, json=payload,
                             converter=models.Broadcast.convert)
 
-    def push_pgn_update(self, broadcast_id, pgn_games, slug='-'):
+    def push_pgn_update(self, broadcast_round_id, pgn_games):
         """Manually update an existing broadcast by ID.
 
-        :param str broadcast_id: ID of a broadcast
+        :param str broadcast_round_id: ID of a broadcast round
         :param list pgn_games: one or more games in PGN format
         :return: success
         :rtype: bool
         """
-        path = f'broadcast/{slug}/{broadcast_id}/push'
+        path = f'broadcast/round/{broadcast_round_id}/push'
         games = '\n\n'.join(g.strip() for g in pgn_games)
         return self._r.post(path, data=games)['ok']
 
+    def create_round(self, broadcast_id, name, sync_url=None, starts_at=None):
+        """Create a new broadcast round to relay external games.
+
+        :param str broadcast_id: broadcast tournament ID
+        :param str name: Name of the broadcast round
+        :param str sync_url: URL that Lichess will poll to get updates about the games.
+        :param int starts_at: Timestamp in milliseconds of broadcast round start
+        :return: success
+        :rtype: dict
+        """
+        path = f'broadcast/{broadcast_id}/new'
+        payload = {
+            'name': name,
+            'syncUrl': sync_url,
+            'startsAt': starts_at
+        }
+        return self._r.post(path, json=payload, converter=models.Broadcast.convert)
+
+    def get_round(self, broadcast_id, broadcast_tournament_slug='-', broadcast_round_slug='-'):
+        """Get information about a broadcast round
+
+        :param broadcast_id: broadcast round id
+        :param str broadcast_tournament_slug: Only used for SEO, can be safely replaced by -
+        :param str broadcast_round_slug: Only used for SEO, can be safely replaced by -
+        :return: broadcast round info
+        :rtype: dict
+        """
+        path = f'broadcast/{broadcast_tournament_slug}/{broadcast_round_slug}/{broadcast_id}'
+        return self._r.get(path, converter=models.Broadcast.convert)
+
+    def update_round(self, broadcast_id, name, sync_url=None, starts_at=None):
+        """Update information about a broadcast round that you created
+
+        :param str broadcast_id: broadcast round id
+        :param str name: Name of the broadcast round
+        :param str sync_url: URL that Lichess will poll to get updates about the games
+        :param starts_at: Timestamp in milliseconds of broadcast start
+        :return: updated broadcast information
+        :rtype: dict
+        """
+        path = f'broadcast/round/{broadcast_id}/edit'
+        payload = {
+            'name': name,
+            'syncUrl': sync_url,
+            'startsAt': starts_at
+        }
+        return self._r.post(path, json=payload,
+                            converter=models.Broadcast.convert)
+
 
 class Simuls(BaseClient):
     """Simultaneous exhibitions - one vs many."""
@@ -1379,3 +1478,18 @@ class Studies(BaseClient):
         """
         path = f'/study/{study_id}.pgn'
         return self._r.get(path, fmt=PGN, stream=True)
+
+
+class Messaging(BaseClient):
+
+    def send(self, username, text):
+        """Send a private message to another player.
+
+        :param str username: the user to send the message to
+        :param str text: the text to send
+        """
+        path = f'/inbox/{username}'
+        payload = {
+            'text': text
+        }
+        self._r.post(path, data=payload)
