@@ -2,7 +2,7 @@
     0. UPDATE SETUP.PY VERSION
     1. PUSH TO GITHUB
     2. DELETE DIST FOLDER
-    3. setup.py sdist bdist_wheel
+    3. python setup.py sdist bdist_wheel
     4. python -m twine upload dist/*
     5. Enter username
     6. Enter password
